import logging

from .const import SENSITIVE_FIELD_NAMES

_LOGGER = logging.getLogger(__name__)


def clean_dictionary_for_logging(dictionary: dict[str, any]) -> dict[str, any]:
    mutable_dictionary = dictionary.copy()
    for key in dictionary.keys():
        if key.lower() in SENSITIVE_FIELD_NAMES:
            mutable_dictionary[key] = "***"
        if type(mutable_dictionary[key]) is dict:
            mutable_dictionary[key] = clean_dictionary_for_logging(
                mutable_dictionary[key].copy()
            )
        if type(mutable_dictionary[key]) is list:
            new_array = []
            for item in mutable_dictionary[key]:
                if type(item) is dict:
                    new_array.append(clean_dictionary_for_logging(item.copy()))
                else:
                    new_array.append(item)
            mutable_dictionary[key] = new_array

    return mutable_dictionary



import logging

from datetime import datetime
import random
import string
import secrets
import json

import pytz
import time

from aiohttp import ClientSession, ClientResponse, ClientError

from .errors import AuthError
from .util_http import request_with_logging

_LOGGER = logging.getLogger(__name__)


class UsKiaWeb:
    def __init__(self, client_session: ClientSession = None):
        # Randomly generate a plausible device id on startup
        self.device_id = (
            "".join(
                random.choice(string.ascii_letters + string.digits) for _ in range(22)
            )
            + ":"
            + secrets.token_urlsafe(105)
        )

        self.BASE_URL: str = "https://owners.kia.com"
        self.API_URL: str = self.BASE_URL + "/apps/services/owners/"

        if client_session is None:
            self.api_session = ClientSession(raise_for_status=True)
        else:
            self.api_session = client_session

    async def cleanup_client_session(self):
        await self.api_session.close()

    @request_with_logging
    async def _post_request_with_logging_and_errors_raised(
        self, headers: dict, url: str, data: dict, json_body: dict
    ) -> ClientResponse:
        return await self.api_session.post(url=url, data=data, json=json_body, headers=headers)

    @request_with_logging
    async def _get_request_with_logging_and_errors_raised(
        self, session_id: str, vehicle_key: str, url: str
    ) -> ClientResponse:
        headers = self._api_headers(session_id, vehicle_key)
        return await self.api_session.get(url=url, headers=headers)

    async def login(self, username, password):
        url = self.API_URL + "apiGateway"
        json_body = {
            "userId": username,
            "password": password,
            "userType": "1",
            "vin": "",
            "action": "authenticateUser"
        }
        headers = {
            "Content-Type": "application/x-www-form-urlencoded"
        }
        response = await self._post_request_with_logging_and_errors_raised(
            url=url,
            json_body=None,
            data=json.dumps(json_body),
            headers=headers
        )
        return response.cookies["JSESSIONID"], await response.json(content_type="utf-8;charset=iso-8859-1")



import logging

from getpass import getpass
from pathlib import Path
import sys

path_root = Path(__file__).parents[2]
sys.path.append(str(path_root))
from src.kia_hyundai_api.ca import Ca

logger = logging.getLogger("src.kia_hyundai_api")
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)


async def testing(ca_api: Ca):
    username = input("Username: ")
    pin = getpass("Pin: ")
    password = getpass()
    try:
        access_token, _ = await ca_api.login(username, password)
        vehicles = await ca_api.get_vehicles(access_token)
        vehicle_id = vehicles["vehicles"][0]["vehicleId"]
        await ca_api.get_cached_vehicle_status(
            access_token=access_token, vehicle_id=vehicle_id
        )
        await ca_api.get_next_service_status(
            access_token=access_token, vehicle_id=vehicle_id
        )
        pin_token = await ca_api.get_pin_token(access_token=access_token, pin=pin)
        await ca_api.get_location(
            access_token=access_token,
            vehicle_id=vehicle_id,
            pin=pin,
            pin_token=pin_token,
        )
        await ca_api.lock(
            access_token=access_token,
            vehicle_id=vehicle_id,
            pin=pin,
            pin_token=pin_token,
        )
    finally:
        await ca_api.cleanup_client_session()



import logging

from .util import clean_dictionary_for_logging

_LOGGER = logging.getLogger(__name__)


def request_with_logging(func):
    async def request_with_logging_wrapper(*args, **kwargs):
        url = kwargs["url"]
        request_message = f"sending {url} request"
        headers = kwargs.get("headers")
        if headers is not None:
            request_message = request_message + f"headers: {headers}"
        json_body = kwargs.get("json_body")
        if json_body is not None:
            request_message = (
                request_message
                + f"sending {url} request with {clean_dictionary_for_logging(json_body)}"
            )
        _LOGGER.debug(request_message)
        response = await func(*args, **kwargs)
        _LOGGER.debug(
            f"response headers:{clean_dictionary_for_logging(response.headers)}"
        )
        _LOGGER.debug(f"response status: {response.status}")
        try:
            response_json = await response.json(content_type=None)
            _LOGGER.debug(
                f"response json: {clean_dictionary_for_logging(response_json)}"
            )
        except Exception:
            response_text = await response.text()
            _LOGGER.debug(f"response raw: {response_text}")
        return response

    return request_with_logging_wrapper



CA_TEMP_RANGE_ICE_C = [x * 0.5 for x in range(32, 64)]
CA_TEMP_RANGE_EV_C = [x * 0.5 for x in range(28, 64)]

SENSITIVE_FIELD_NAMES = [
    "username",
    "password",
    "userid",
    "vin",
    "sid",
    "vinkey",
    "lat",
    "lon",
]



import logging

import time
from abc import ABC, abstractmethod
from aiohttp import ClientSession, ClientResponse, ClientError

from .errors import RateError, AuthError
from .const import CA_TEMP_RANGE_ICE_C, CA_TEMP_RANGE_EV_C
from .util import clean_dictionary_for_logging

_LOGGER = logging.getLogger(__name__)


def request_with_logging_and_errors_raised(func):
    """
    {'error': {'errorCode': '6533', 'errorDesc': 'We are currently processing an earlier inquiry for your vehicle. Please wait 90 seconds before issuing next request.'}, 'responseHeader': {'responseCode': 1, 'responseDesc': 'Failure'}}
    {'error': {'errorCode': '7310', 'errorDesc': 'PIN incorrect. Multiple incorrect attempts will require PIN reset'}, 'responseHeader': {'responseCode': 1, 'responseDesc': 'Failure'}}
    {'error': {'errorCode': '6534', 'errorDesc': 'We apologize, but your daily limit of remote services requests has been exceeded. Your request for <<service name>> was not processed. Please try again tomorrow.'}, 'responseHeader': {'responseCode': 1, 'responseDesc': 'Failure'}}
    """

    async def request_with_logging_wrapper(*args, **kwargs):
        url = kwargs["url"]
        json_body = kwargs.get("json_body")
        if json_body is not None:
            _LOGGER.debug(
                f"sending {url} request with {clean_dictionary_for_logging(json_body)}"
            )
        else:
            _LOGGER.debug(f"sending {url} request")
        response = await func(*args, **kwargs)
        _LOGGER.debug(
            f"response headers:{clean_dictionary_for_logging(response.headers)}"
        )
        try:
            response_json = await response.json()
            _LOGGER.debug(
                f"response json:{clean_dictionary_for_logging(response_json)}"
            )
        except RuntimeError:
            response_text = await response.text()
            _LOGGER.debug(f"response text:{response_text}")
        if response_json["responseHeader"]["responseCode"] == 0:
            return response
        # still need to add error code for expired token
        if response_json["error"]["errorCode"] == "7404":
            _LOGGER.error(f"invalid password")
            raise AuthError(response_json["error"]["errorDesc"])
        if response_json["error"]["errorCode"] == "7310":
            _LOGGER.error(f"invalid pin")
            raise AuthError(response_json["error"]["errorDesc"])
        if response_json["error"]["errorCode"] in ["6533", "6534"]:
            raise RateError(response_json["error"]["errorDesc"])
        response_text = await response.text()
        _LOGGER.error(f"error: unknown error response {response_text}")
        raise ClientError

    return request_with_logging_wrapper


class Ca(ABC):
    def __init__(self, client_session: ClientSession = None):
        if client_session is None:
            self.api_session = ClientSession(raise_for_status=True)
        else:
            self.api_session = client_session

    async def cleanup_client_session(self):
        await self.api_session.close()

    @property
    @abstractmethod
    def base_url(self):
        pass

    @property
    def api_url(self):
        return "https://" + self.base_url + "/tods/api/"

    def _api_headers(
        self,
        access_token: str = None,
        vehicle_id: str = None,
        pin_token: str = None,
        xid: str = None,
    ) -> dict:
        headers = {
            "content-type": "application/json;charset=UTF-8",
            "accept": "application/json, text/plain, */*",
            "accept-encoding": "gzip, deflate, br",
            "accept-language": "en-US,en;q=0.9",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36",
            "host": self.base_url,
            "origin": "https://" + self.base_url,
            "referer": "https://" + self.base_url + "/login",
            "from": "SPA",
            "language": "0",
            "offset": str(int(time.localtime().tm_gmtoff / 60 / 60)),
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
        }
        if access_token is not None:
            headers["accessToken"] = access_token
        if vehicle_id is not None:
            headers["vehicleId"] = vehicle_id
        if pin_token is not None:
            headers["pAuth"] = pin_token
        if xid is not None:
            headers["transactionId"] = xid

        return headers

    @request_with_logging_and_errors_raised
    async def _post_request_with_logging_and_errors_raised(
        self,
        access_token: str,
        vehicle_id: str,
        url: str,
        json_body: dict = None,
        pin_token: str = None,
        xid: str = None,
    ) -> ClientResponse:
        if json_body is not None:
            pin = json_body.get("pin")
            if pin is not None and pin_token is None and not url.endswith("vrfypin"):
                pin_token_response = await self.get_pin_token(access_token, pin)
                pin_token = pin_token_response["pAuth"]

        headers = self._api_headers(access_token, vehicle_id, pin_token, xid)
        return await self.api_session.post(url=url, json=json_body, headers=headers)

    async def login(self, username: str, password: str) -> (str, str):
        url = self.api_url + "lgn"
        json_body = {"loginId": username, "password": password}
        response: ClientResponse = (
            await self._post_request_with_logging_and_errors_raised(
                access_token=None, vehicle_id=None, url=url, json_body=json_body
            )
        )
        response_json = await response.json()
        access_token = response_json["result"]["accessToken"]
        refresh_token = response_json["result"]["refreshToken"]
        return access_token, refresh_token

    async def get_vehicles(self, access_token: str):
        """
        {"responseHeader":{"responseCode":0,"responseDesc":"Success"},"result":{"vehicles":[{"vehicleId":"ID","nickName":"Optimus","modelCode":"D5S6D2A1FG","modelName":"Optima","modelYear":"2019","fuelKindCode":"G","trim":"","engine":"","exteriorColor":"M5U","dtcCount":0,"subscriptionStatus":"A","subscriptionEndDate":"20240422","mileageForNextService":-43557.8,"daysForNextService":"20200427","overviewMessage":"","defaultVehicle":true,"enrollmentStatus":"4","genType":"G1","vin":"7986"}]}}
        """
        url = self.api_url + "vhcllst"
        response: ClientResponse = (
            await self._post_request_with_logging_and_errors_raised(
                access_token=access_token, vehicle_id=None, url=url
            )
        )
        response_json = await response.json()
        return response_json["result"]

    async def get_cached_vehicle_status(self, access_token: str, vehicle_id: str):
        """
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}, 'result': {'status': {'lastStatusDate': '20211119145414', 'airCtrlOn': False, 'engine': False, 'doorLock': True, 'doorOpen': {'frontLeft': 0, 'frontRight': 0, 'backLeft': 0, 'backRight': 0}, 'trunkOpen': False, 'airTempUnit': 'C', 'airTemp': {'value': '01H', 'unit': 0}, 'defrost': False, 'lowFuelLight': False, 'acc': False, 'hoodOpen': False, 'transCond': True, 'steerWheelHeat': 0, 'sideBackWindowHeat': 0, 'dte': {'value': 416.0, 'unit': 1}, 'tirePressureLamp': {'tirePressureLampAll': 0, 'tirePressureLampFL': 0, 'tirePressureLampFR': 0, 'tirePressureLampRL': 0, 'tirePressureLampRR': 0}, 'battery': {'batSoc': 81, 'batState': 0, 'sjbDeliveryMode': 0, 'batSignalReferenceValue': {}}, 'remoteIgnition': True, 'seatHeaterVentInfo': {'drvSeatHeatState': 2, 'astSeatHeatState': 2, 'rlSeatHeatState': 2, 'rrSeatHeatState': 2}, 'sleepModeCheck': True, 'lampWireStatus': {'headLamp': {}, 'stopLamp': {}, 'turnSignalLamp': {}}, 'windowOpen': {}, 'engineOilStatus': False, 'vehicleMovementHis': True, 'engineRuntime': {'value': 29, 'unit': 1}}}}
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}, 'result': {'status': {'lastStatusDate': '20210801171424', 'airCtrlOn': False, 'engine': False, 'doorLock': False, 'doorOpen': {'frontLeft': 0, 'frontRight': 0, 'backLeft': 0, 'backRight': 0}, 'trunkOpen': False, 'airTempUnit': 'C', 'airTemp': {'value': '01H', 'unit': 0}, 'defrost': False, 'lowFuelLight': False, 'acc': False, 'hoodOpen': False, 'transCond': True, 'dte': {'unit': 1}, 'tirePressureLamp': {}, 'battery': {'batSoc': 77, 'batSignalReferenceValue': {}}, 'remoteIgnition': True, 'seatHeaterVentInfo': {}, 'sleepModeCheck': True, 'lampWireStatus': {'headLamp': {}, 'stopLamp': {}, 'turnSignalLamp': {}}, 'windowOpen': {}, 'engineRuntime': {}}}}
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}, 'result': {'status': {'lastStatusDate': '20210920003648', 'airCtrlOn': False, 'engine': False, 'doorLock': False, 'doorOpen': {'frontLeft': 0, 'frontRight': 0, 'backLeft': 0, 'backRight': 0}, 'trunkOpen': False, 'airTempUnit': 'C', 'airTemp': {'value': '01H', 'unit': 0}, 'defrost': False, 'lowFuelLight': False, 'acc': False, 'hoodOpen': False, 'transCond': True, 'steerWheelHeat': 0, 'sideBackWindowHeat': 0, 'dte': {}, 'tirePressureLamp': {}, 'battery': {'batSoc': 81, 'batSignalReferenceValue': {}}, 'remoteIgnition': True, 'seatHeaterVentInfo': {}, 'sleepModeCheck': True, 'lampWireStatus': {'headLamp': {}, 'stopLamp': {}, 'turnSignalLamp': {}}, 'windowOpen': {}, 'engineRuntime': {}}}}
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}, 'result': {'status': {'lastStatusDate': '20211116153356', 'airCtrlOn': False, 'engine': False, 'doorLock': True, 'doorOpen': {'frontLeft': 0, 'frontRight': 0, 'backLeft': 0, 'backRight': 0}, 'trunkOpen': False, 'airTempUnit': 'C', 'airTemp': {'value': '00H', 'unit': 0}, 'defrost': False, 'acc': False, 'evStatus': {'batteryCharge': False, 'batteryStatus': 29, 'batteryPlugin': 0, 'remainTime2': { 'etc1': { 'value': 0, 'unit': 1 }, 'etc2': { 'value': 0, 'unit': 1 }, 'etc3': { 'value': 0, 'unit': 1 }, 'atc': { 'value': 0, 'unit': 1 } }, 'drvDistance': [ { 'rangeByFuel': { 'gasModeRange': { 'value': 0.0, 'unit': 1 }, 'evModeRange': { 'value': 104.0, 'unit': 1 }, 'totalAvailableRange': { 'value': 104.0, 'unit': 1 } }, 'type': 2 } ] }, 'hoodOpen': False, 'transCond': True, 'steerWheelHeat': 0, 'sideBackWindowHeat': 0, 'dte': { }, 'tirePressureLamp': { 'tirePressureLampAll': 0 }, 'battery': { 'batSoc': 87, 'batState': 0, 'batSignalReferenceValue': { } }, 'remoteIgnition': True, 'seatHeaterVentInfo': { }, 'sleepModeCheck': False, 'lampWireStatus': { 'headLamp': { }, 'stopLamp': { }, 'turnSignalLamp': { } }, 'windowOpen': { }, 'engineRuntime': { } } } }
        """
        url = self.api_url + "lstvhclsts"
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token, vehicle_id=vehicle_id, url=url
        )
        response_json = await response.json()
        return response_json["result"]

    async def get_next_service_status(self, access_token: str, vehicle_id: str):
        """
        { "responseHeader": { "responseCode": 0, "responseDesc": "Success" }, "result": { "maintenanceInfo": { "msopServiceOdometer": 36057.0, "msopServiceOdometerUnit": 1, "mtspServiceDate": "20210721", "imatServiceOdometer": 48057.0, "imatServiceOdometerUnit": 1, "mtitServiceDate": "20220121", "currentOdometer": 37236.8, "currentOdometerUnit": 1, "serviceOdometerDuration": 12000.0, "serviceDaysDuration": 184, "serviceMonthsThreshold": 6 } } }
        { 'responseHeader': { 'responseCode': 0, 'responseDesc': 'Success' }, 'result': { 'maintenanceInfo': { 'mtspServiceDate': '20190427', 'imatServiceOdometer': 12000.0, 'imatServiceOdometerUnit': 1, 'mtitServiceDate': '20200427', 'currentOdometer': 58322.9, 'currentOdometerUnit': 1, 'serviceOdometerDuration': 12000.0, 'serviceDaysDuration': 366, 'serviceMonthsThreshold': 12 } } }
        { 'responseHeader': { 'responseCode': 0, 'responseDesc': 'Success' }, 'result': { 'maintenanceInfo': { 'mtspServiceDate': '20210917', 'imatServiceOdometer': 6000.0, 'imatServiceOdometerUnit': 1, 'mtitServiceDate': '20220317', 'currentOdometer': 2213.899902, 'currentOdometerUnit': 1, 'serviceOdometerDuration': 6000.0, 'serviceDaysDuration': 181, 'serviceMonthsThreshold': 6 } } }
        """
        url = self.api_url + "nxtsvc"
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token, vehicle_id=vehicle_id, url=url
        )
        response_json = await response.json()
        return response_json["result"]

    async def get_pin_token(self, access_token: str, pin: str) -> dict[str, any]:
        url = self.api_url + "vrfypin"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token, vehicle_id=None, url=url, json_body=json_body
        )
        response_json = await response.json()
        return response_json["result"]

    async def get_location(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ):
        url = self.api_url + "fndmcr"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            pin_token=pin_token,
            vehicle_id=vehicle_id,
            url=url,
            json_body=json_body,
        )
        response_json = await response.json()
        return response_json["result"]

    async def request_vehicle_data_sync(self, access_token: str, vehicle_id: str):
        """
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}, 'result': {'status': {'lastStatusDate': '20211119221003', 'airCtrlOn': False, 'engine': False, 'doorLock': True, 'doorOpen': {'frontLeft': 0, 'frontRight': 0, 'backLeft': 0, 'backRight': 0}, 'trunkOpen': False, 'airTempUnit': 'C', 'airTemp': {'value': '01H', 'unit': 0}, 'defrost': False, 'lowFuelLight': False, 'acc': False, 'hoodOpen': False, 'transCond': True, 'steerWheelHeat': 0, 'sideBackWindowHeat': 0, 'dte': {'value': 416.0, 'unit': 1}, 'tirePressureLamp': {'tirePressureLampAll': 0, 'tirePressureLampFL': 0, 'tirePressureLampFR': 0, 'tirePressureLampRL': 0, 'tirePressureLampRR': 0}, 'battery': {'batSoc': 81, 'batState': 0, 'sjbDeliveryMode': 0, 'batSignalReferenceValue': {}}, 'remoteIgnition': True, 'seatHeaterVentInfo': {'drvSeatHeatState': 2, 'astSeatHeatState': 2, 'rlSeatHeatState': 2, 'rrSeatHeatState': 2}, 'lampWireStatus': {'headLamp': {}, 'stopLamp': {}, 'turnSignalLamp': {}}, 'windowOpen': {}, 'engineOilStatus': False, 'vehicleMovementHis': False, 'engineRuntime': {'value': 0, 'unit': 1}}}}
        """
        url = self.api_url + "rltmvhclsts"
        await self._post_request_with_logging_and_errors_raised(
            access_token=access_token, vehicle_id=vehicle_id, url=url
        )

    async def check_last_action_status(
        self,
        access_token: str,
        vehicle_id: str,
        pin: str,
        xid: str,
        pin_token: str = None,
    ) -> dict:
        url = self.api_url + "rmtsts"

        if pin_token is None:
            pin_token_response = await self.get_pin_token(access_token, pin)
            pin_token = pin_token_response["pAuth"]

        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            xid=xid,
        )
        response_json = await response.json()
        return response_json["result"]

    async def lock(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ) -> str:
        """
        {'responseHeader': {'responseCode': 0, 'responseDesc': 'Success'}}
        """
        url = self.api_url + "drlck"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def unlock(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ) -> str:
        url = self.api_url + "drulck"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def start_climate(
        self,
        access_token: str,
        vehicle_id: str,
        pin: str,
        set_temp,
        duration,
        defrost,
        climate,
        heating,
        pin_token: str = None,
    ):
        url = self.api_url + "rmtstrt"
        api_set_temp = CA_TEMP_RANGE_ICE_C.index(set_temp)
        api_set_temp = hex(api_set_temp).split("x")
        api_set_temp = api_set_temp[1] + "H"
        api_set_temp = api_set_temp.zfill(3).upper()

        json_body = {
            "setting": {
                "airCtrl": int(climate),
                "defrost": defrost,
                "heating1": int(heating),
                "igniOnDuration": duration,
                "ims": 0,
                "airTemp": {
                    "value": api_set_temp,
                    "unit": 0,
                    "hvacTempType": 0,
                },
            },
            "pin": pin,
        }

        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def start_climate_ev(
        self,
        access_token: str,
        vehicle_id: str,
        pin: str,
        set_temp,
        defrost,
        climate,
        heating,
        pin_token: str = None,
    ):
        url = self.api_url + "evc/rfon"
        api_set_temp = CA_TEMP_RANGE_EV_C.index(set_temp)
        api_set_temp = hex(api_set_temp).split("x")
        api_set_temp = api_set_temp[1] + "H"
        api_set_temp = api_set_temp.zfill(3).upper()

        json_body = {
            "hvacInfo": {
                "airCtrl": int(climate),
                "defrost": defrost,
                "heating1": int(heating),
                "airTemp": {
                    "value": api_set_temp,
                    "unit": 0,
                    "hvacTempType": 1,
                },
            },
            "pin": pin,
        }
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def stop_climate(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ):
        url = self.api_url + "rmtstp"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def stop_climate_ev(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ):
        url = self.api_url + "evc/rfoff"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def start_charge(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ):
        url = self.api_url + "evc/rcstrt"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]

    async def stop_charge(
        self, access_token: str, vehicle_id: str, pin: str, pin_token: str = None
    ):
        url = self.api_url + "evc/rcstp"
        json_body = {"pin": pin}
        response = await self._post_request_with_logging_and_errors_raised(
            access_token=access_token,
            vehicle_id=vehicle_id,
            url=url,
            pin_token=pin_token,
            json_body=json_body,
        )
        return response.headers["transactionId"]



# run with "python3 src/kia_hyundai_api/us_hyundai_stub.py"
import logging
import asyncio

from getpass import getpass
from pathlib import Path
import sys

path_root = Path(__file__).parents[2]
sys.path.append(str(path_root))
from src.kia_hyundai_api import UsHyundai

logger = logging.getLogger("src.kia_hyundai_api")
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)


async def testing():
    api = UsHyundai()
    username = input("Username: ")
    pin = getpass("Pin: ")
    password = getpass()
    try:
        access_token, refresh_token, expires_in = await api.login(
            username=username, password=password, pin=pin
        )
        vehicles = await api.get_vehicles(
            username=username, pin=pin, access_token=access_token
        )
        vin = vehicles["enrolledVehicleDetails"][0]["vehicleDetails"]["vin"]
        reg_id = vehicles["enrolledVehicleDetails"][0]["vehicleDetails"]["regid"]
        await api.get_cached_vehicle_status(
            username=username, pin=pin, access_token=access_token, vehicle_vin=vin
        )
        await api.get_location(
            username=username, pin=pin, access_token=access_token, vehicle_vin=vin
        )
        await api.lock(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vin,
            vehicle_regid=reg_id,
        )
    finally:
        await api.cleanup_client_session()


asyncio.run(testing())



import logging

from datetime import datetime
import random
import string
import secrets
import ssl
import certifi

import pytz
import time

from aiohttp import ClientSession, ClientResponse, ClientError

from .errors import AuthError
from .util import clean_dictionary_for_logging

_LOGGER = logging.getLogger(__name__)


def request_with_logging(func):
    async def request_with_logging_wrapper(*args, **kwargs):
        url = kwargs["url"]
        json_body = kwargs.get("json_body")
        if json_body is not None:
            _LOGGER.debug(
                f"sending {url} request with {clean_dictionary_for_logging(json_body)}"
            )
        else:
            _LOGGER.debug(f"sending {url} request")
        response = await func(*args, **kwargs)
        _LOGGER.debug(
            f"response headers:{clean_dictionary_for_logging(response.headers)}"
        )
        try:
            response_json = await response.json()
            _LOGGER.debug(
                f"response json:{clean_dictionary_for_logging(response_json)}"
            )
        except RuntimeError:
            response_text = await response.text()
            _LOGGER.debug(f"response text:{response_text}")
        if response_json["status"]["statusCode"] == 0:
            return response
        if (
            response_json["status"]["statusCode"] == 1
            and response_json["status"]["errorType"] == 1
            and (
                response_json["status"]["errorCode"] == 1001
                or response_json["status"]["errorCode"] == 1003
                or response_json["status"]["errorCode"] == 1005
                or response_json["status"]["errorCode"] == 1037
            )
        ):
            _LOGGER.debug(f"error: session invalid")
            raise AuthError
        response_text = await response.text()
        _LOGGER.debug(f"error: unknown error response {response_text}")
        raise ClientError(f"api error:{response_json['status']['errorMessage']}")

    return request_with_logging_wrapper


class UsKia:
    def __init__(self, client_session: ClientSession = None):
        # Randomly generate a plausible device id on startup
        self.device_id = (
            "".join(
                random.choice(string.ascii_letters + string.digits) for _ in range(22)
            )
            + ":"
            + secrets.token_urlsafe(105)
        )

        self.BASE_URL: str = "api.owners.kia.com"
        self.API_URL: str = "https://" + self.BASE_URL + "/apigw/v1/"

        if client_session is None:
            self.api_session = ClientSession(raise_for_status=True)
        else:
            self.api_session = client_session

        new_ssl_context = ssl.create_default_context(cafile=certifi.where())
        new_ssl_context.load_default_certs()
        new_ssl_context.check_hostname = True
        new_ssl_context.verify_mode = ssl.CERT_REQUIRED
        new_ssl_context.set_ciphers("ALL")
        new_ssl_context.options = (
                ssl.OP_CIPHER_SERVER_PREFERENCE
        )
        self.ssl_context = new_ssl_context

    async def cleanup_client_session(self):
        await self.api_session.close()

    def _api_headers(self, session_id: str = None, vehicle_key: str = None) -> dict:
        headers = {
            "content-type": "application/json;charset=UTF-8",
            "accept": "application/json, text/plain, */*",
            "accept-encoding": "gzip, deflate, br",
            "accept-language": "en-US,en;q=0.9",
            "apptype": "L",
            "appversion": "4.10.0",
            "clientid": "MWAMOBILE",
            "from": "SPA",
            "host": self.BASE_URL,
            "language": "0",
            "offset": str(int(time.localtime().tm_gmtoff / 60 / 60)),
            "ostype": "Android",
            "osversion": "11",
            "secretkey": "98er-w34rf-ibf3-3f6h",
            "to": "APIGW",
            "tokentype": "G",
            "user-agent": "okhttp/3.12.1",
            "deviceid": self.device_id,
            "date": datetime.now(tz=pytz.utc).strftime("%a, %d %b %Y %H:%M:%S GMT"),
        }
        if session_id is not None:
            headers["sid"] = session_id
        if vehicle_key is not None:
            headers["vinkey"] = vehicle_key
        return headers

    @request_with_logging
    async def _post_request_with_logging_and_errors_raised(
        self, session_id: str, vehicle_key: str, url: str, json_body: dict
    ) -> ClientResponse:
        headers = self._api_headers(session_id, vehicle_key)
        return await self.api_session.post(
            url=url,
            json=json_body,
            headers=headers,
            ssl=self.ssl_context
        )

    @request_with_logging
    async def _get_request_with_logging_and_errors_raised(
        self, session_id: str, vehicle_key: str, url: str
    ) -> ClientResponse:
        headers = self._api_headers(session_id, vehicle_key)
        return await self.api_session.get(
            url=url,
            headers=headers,
            ssl=self.ssl_context
        )

    async def login(self, username: str, password: str) -> str:
        url = self.API_URL + "prof/authUser"

        data = {
            "deviceKey": "",
            "deviceType": 2,
            "userCredential": {"userId": username, "password": password},
        }
        response: ClientResponse = (
            await self._post_request_with_logging_and_errors_raised(
                session_id=None, vehicle_key=None, url=url, json_body=data
            )
        )
        session_id = response.headers.get("sid")
        if not session_id:
            response_text = await response.text()
            raise Exception(
                f"no session id returned in login. Response: {response_text} headers {response.headers} cookies {response.cookies}"
            )
        _LOGGER.debug(f"got session id {session_id}")
        return session_id

    async def get_vehicles(self, session_id: str):
        """
        {"status":{"statusCode":0,"errorType":0,"errorCode":0,"errorMessage":"Success with response body"},"payload":{"vehicleSummary":[{"vin":"VIN","vehicleIdentifier":"1234","modelName":"NIRO EV","modelYear":"2019","nickName":"Niro EV","generation":2,"extColorCode":"C3S","trim":"EX PREMIUM","imagePath":{"imageName":"2019-niro_ev-ex_premium-c3s.png","imagePath":"/content/dam/kia/us/owners/image/vehicle/2019/niro_ev/ex_premium/","imageType":"1","imageSize":{"length":"100","width":"100","uom":0}},"enrollmentStatus":1,"fatcAvailable":1,"telematicsUnit":1,"fuelType":4,"colorName":"ALUMINUM SILVER","activationType":2,"mileage":"12844.7","dealerCode":"MD047","mobileStore":[{"osType":0,"downloadURL":"https://itunes.apple.com/us/app/kia-access-with-uvo-link/id1280548773?mt=8","image":{"imageName":"iosImage.png","imagePath":"/content/dam/kia/us/owners/image/common/app/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}},{"osType":1,"downloadURL":"https://play.google.com/store/apps/details?id=com.myuvo.link","image":{"imageName":"androidImage.png","imagePath":"/content/dam/kia/us/owners/image/common/app/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}}],"supportedApp":{"appType":"5","appImage":{"imageName":"uvo-app.png","imagePath":"/content/dam/kia/us/owners/image/common/app/access/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}},"supportAdditionalDriver":0,"customerType":0,"projectCode":"DEEV","headUnitDesc":"AVN5.0","provStatus":"4","enrollmentSuppressionType":0,"vehicleKey":"KEY"}]}}
        """
        url = self.API_URL + "ownr/gvl"
        response: ClientResponse = (
            await self._get_request_with_logging_and_errors_raised(
                session_id=session_id, vehicle_key=None, url=url
            )
        )
        response_json = await response.json()
        return response_json["payload"]

    async def get_cached_vehicle_status(self, session_id: str, vehicle_key: str):
        """
        {"status":{"statusCode":0,"errorType":0,"errorCode":0,"errorMessage":"Success with response body"},"payload":{"vehicleInfoList":[{"vinKey":"KEY","vehicleConfig":{"vehicleDetail":{"vehicle":{"vin":"VIN","trim":{"modelYear":"2019","salesModelCode":"V1262","optionGroupCode":"015","modelName":"NIRO EV","factoryCode":"DQ","projectCode":"DEEV","trimName":"EX PREMIUM","driveType":"0","transmissionType":"1","ivrCategory":"6","btSeriesCode":"N"},"telematics":1,"mileage":"13228.1","mileageSyncDate":"20211211075445","exteriorColor":"ALUMINUM SILVER","exteriorColorCode":"C3S","fuelType":4,"invDealerCode":"MD047","testVehicle":"0","supportedApps":[{"appType":"0"},{"appType":"5","appImage":{"imageName":"uvo-app.png","imagePath":"/content/dam/kia/us/owners/image/common/app/access/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}}],"activationType":2},"images":[{"imageName":"2019-niro_ev-ex_premium-c3s.png","imagePath":"/content/dam/kia/us/owners/image/vehicle/2019/niro_ev/ex_premium/","imageType":"1","imageSize":{"length":"100","width":"100","uom":0}}],"device":{"launchType":"0","swVersion":"DEEV.USA.SOP.V105.190503.STD_H","telematics":{"generation":"3","platform":"1","tmsCenter":"1","billing":true},"versionNum":"ECO","headUnitType":"0","hdRadio":"X40HA","ampType":"NA","modem":{"meid":"MEID","mdn":"MDN","iccid":"ICCID"},"headUnitName":"avn40ev_np","bluetoothRef":"10","headUnitDesc":"AVN5.0"}},"maintenance":{"nextServiceMile":1771.9004,"maintenanceSchedule":[6500,7500,13000,15000,19500,22500,26000,30000,32500,37500,39000,45000,45500,52000,52500,58500,60000,65000,67500,71500,75000,78000,82500,84500,90000,91000,97500,104000,105000,110500,112500]},"vehicleFeature":{"remoteFeature":{"lock":"1","unlock":"1","start":"3","stop":"1","scheduleCount":"2","inVehicleSchedule":"1","heatedSteeringWheel":"1","heatedSideMirror":"1","heatedRearWindow":"1","heatedSeat":"0","ventSeat":"0","alarm":"1","hornlight":"1","panic":"1","separateHeatedAccessories":"0","windowSafety":"0"},"chargeFeature":{"batteryChargeType":"1","chargeEndPct":"4","immediateCharge":"1","cancelCharge":"1","evRange":"1","scheduleCount":"2","inVehicleSchedule":"1","offPeakType":"2","scheduleType":"2","chargeLevel":"3","scheduleConfig":"1","fatcWithCharge":"1","targetSOC":"1","minTargetSOC":"50","maxTargetSOC":"100","socStep":"10"},"alertFeature":{"geofenceType":{"geofence":"1","entryCount":"5","exitCount":"1","inVehicleConfig":"0","minRadius":"1","maxRadius":"10","minHeight":"1","maxHeight":"10","minWidth":"1","maxWidth":"10","uom":"0"},"curfewType":{"curfew":"1","curfewCount":"21","inVehicleConfig":"0"},"speedType":{"speed":"1","speedCount":"21","inVehicleConfig":"0"},"valetType":{"valet":"1","valetParkingMode":"0","defaultRadius":"1","defaultRadiusUnit":"3","defaultInterval":"5","defaultIntervalUnit":"3","inVehicleConfig":"0"}},"vrmFeature":{"autoDTC":"1","scheduledDTC":"1","backgroundDTC":"1","manualDTC":"1","healthReport":"0","drivingScore":"1","gasRange":"0","evRange":"1","trip":"1"},"locationFeature":{"gpsStreaming":"0","location":"1","poi":"1","poiCount":"25","push2Vehicle":"1","wayPoint":"1","mapType":"1","surroundView":"0","svr":"1"},"userSettingFeature":{"usmType":"0","vehicleOptions":"0","systemOptions":"0","additionalDriver":"0","calendar":"0","valetParkingMode":"0","wifiHotSpot":"0","otaSupport":"0"}},"heatVentSeat":{},"billingPeriod":{"freeTrial":{"value":12,"unit":0},"freeTrialExtension":{"value":12,"unit":1},"servicePeriod":{"value":60,"unit":1}}},"lastVehicleInfo":{"vehicleNickName":"Niro EV","preferredDealer":"MD047","customerType":0,"vehicleStatusRpt":{"statusType":"2","reportDate":{"utc":"20211212004604","offset":-8},"vehicleStatus":{"climate":{"airCtrl":false,"defrost":false,"airTemp":{"value":"72","unit":1},"heatingAccessory":{"steeringWheel":0,"sideMirror":0,"rearWindow":0}},"engine":false,"doorLock":true,"doorStatus":{"frontLeft":0,"frontRight":0,"backLeft":0,"backRight":0,"trunk":0,"hood":0},"lowFuelLight":false,"evStatus":{"batteryCharge":false,"batteryStatus":79,"batteryPlugin":0,"remainChargeTime":[{"remainChargeType":2,"timeInterval":{"value":0,"unit":4}},{"remainChargeType":3,"timeInterval":{"value":0,"unit":4}},{"remainChargeType":1,"timeInterval":{"value":0,"unit":4}}],"drvDistance":[{"type":2,"rangeByFuel":{"evModeRange":{"value":214,"unit":3},"totalAvailableRange":{"value":214,"unit":3}}}],"syncDate":{"utc":"20211211225859","offset":-8},"targetSOC":[{"plugType":0,"targetSOClevel":80,"dte":{"type":2,"rangeByFuel":{"gasModeRange":{"value":0,"unit":3},"evModeRange":{"value":214,"unit":3},"totalAvailableRange":{"value":214,"unit":3}}}},{"plugType":1,"targetSOClevel":90,"dte":{"type":2,"rangeByFuel":{"gasModeRange":{"value":0,"unit":3},"evModeRange":{"value":214,"unit":3},"totalAvailableRange":{"value":214,"unit":3}}}}]},"ign3":true,"transCond":true,"tirePressure":{"all":0},"dateTime":{"utc":"20211212004604","offset":-8},"syncDate":{"utc":"20211211225859","offset":-8},"batteryStatus":{"stateOfCharge":81,"sensorStatus":0},"sleepMode":false,"lampWireStatus":{"headLamp":{},"stopLamp":{},"turnSignalLamp":{}},"windowStatus":{},"engineRuntime":{},"valetParkingMode":0}},"location":{"coord":{"lat":1,"lon":-7,"alt":118,"type":0,"altdo":0},"head":349,"speed":{"value":0,"unit":1},"accuracy":{"hdop":6,"pdop":11},"syncDate":{"utc":"20211211225445","offset":-8}},"financed":true,"financeRegistered":true,"linkStatus":0}}]}}
        {"status":{"statusCode":0,"errorType":0,"errorCode":0,"errorMessage":"Success with response body"},"payload":{"vehicleInfoList":[{"vinKey":"KEY","vehicleConfig":{"vehicleDetail":{"vehicle":{"vin":"VIN","trim":{"modelYear":"2021","salesModelCode":"45482","optionGroupCode":"010","modelName":"SPORTAGE","factoryCode":"D9","projectCode":"QL","trimName":"SX-P","driveType":"2","transmissionType":"1","ivrCategory":"5","btSeriesCode":"4"},"telematics":1,"mileage":"11665.9","mileageSyncDate":"20211216143655","exteriorColor":"STEEL GRAY","exteriorColorCode":"KLG","fuelType":1,"invDealerCode":"NJ074","testVehicle":"0","supportedApps":[{"appType":"0"},{"appType":"5","appImage":{"imageName":"uvo-app.png","imagePath":"/content/dam/kia/us/owners/image/common/app/access/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}}],"activationType":2},"images":[{"imageName":"2021-sportage-sx-p-klg.png","imagePath":"/content/dam/kia/us/owners/image/vehicle-app/2021/sportage/sx-p/","imageType":"1","imageSize":{"length":"100","width":"100","uom":0}}],"device":{"launchType":"0","swVersion":"QL21.USA.SOP.V115.200325.STD_H","telematics":{"generation":"3","platform":"1","tmsCenter":"1","billing":true},"versionNum":"GASOLINE","headUnitType":"0","hdRadio":"X40HA","ampType":"NA","modem":{"meid":"MEID","mdn":"MDN","iccid":"ICCID"},"headUnitName":"avn5em","bluetoothRef":"10","headUnitDesc":"AVN5.0"}},"billingPeriod":{"freeTrial":{"value":12,"unit":0},"freeTrialExtension":{"value":12,"unit":1},"servicePeriod":{"value":60,"unit":1}}},"lastVehicleInfo":{"vehicleNickName":"Kia","preferredDealer":"NJ074","customerType":0,"enrollment":{"provStatus":"4","enrollmentStatus":"1","enrollmentType":"0","registrationDate":"20200829","expirationDate":"20210829","expirationMileage":"100000","freeServiceDate":{"startDate":"20200829","endDate":"20210829"}},"activeDTC":{"dtcActiveCount":"0"},"vehicleStatusRpt":{"statusType":"2","reportDate":{"utc":"20211217151540","offset":-8},"vehicleStatus":{"climate":{"airCtrl":false,"defrost":false,"airTemp":{"value":"LOW","unit":1},"heatingAccessory":{"steeringWheel":0,"sideMirror":0,"rearWindow":0}},"engine":false,"doorLock":true,"doorStatus":{"frontLeft":0,"frontRight":0,"backLeft":0,"backRight":0,"trunk":0,"hood":0},"lowFuelLight":false,"ign3":false,"transCond":true,"dateTime":{"utc":"20211217151540","offset":-8},"syncDate":{"utc":"20211217053655","offset":-8},"batteryStatus":{},"sleepMode":true,"lampWireStatus":{"headLamp":{},"stopLamp":{},"turnSignalLamp":{}},"windowStatus":{},"vehicleMovementHis":true,"engineRuntime":{"value":1509,"unit":3},"valetParkingMode":0}},"location":{"coord":{"lat":40.62193333,"lon":-74.4951805556,"alt":113,"type":0,"altdo":0},"head":22,"speed":{"value":0,"unit":1},"accuracy":{"hdop":7,"pdop":13},"syncDate":{"utc":"20211217053655","offset":-8}},"financed":true,"financeRegistered":true,"linkStatus":0}}]}}
        {"status":{"statusCode":0,"errorType":0,"errorCode":0,"errorMessage":"Success with response body"},"payload":{"vehicleInfoList":[{"vinKey":"KEY","vehicleConfig":{"vehicleDetail":{"vehicle":{"vin":"VIN","trim":{"modelYear":"2020","salesModelCode":"G4262","optionGroupCode":"010","modelName":"NIRO","factoryCode":"G5","projectCode":"DEHEV","trimName":"TOURING","driveType":"0","transmissionType":"1","ivrCategory":"5","btSeriesCode":"G"},"telematics":1,"mileage":"0","mileageSyncDate":"20211228211444","exteriorColor":"DEEP CERULEAN","exteriorColorCode":"C3U","fuelType":3,"invDealerCode":"PA004","testVehicle":"0","supportedApps":[{"appType":"0"},{"appType":"5","appImage":{"imageName":"uvo-app.png","imagePath":"/content/dam/kia/us/owners/image/common/app/access/","imageType":"2","imageSize":{"length":"100","width":"100","uom":0}}}],"activationType":2},"images":[{"imageName":"2020-niro-touring-c3u.png","imagePath":"/content/dam/kia/us/owners/image/vehicle-app/2020/niro/touring/","imageType":"1","imageSize":{"length":"100","width":"100","uom":0}}],"device":{"launchType":"0","swVersion":"DEPE_HEV.USA.D2V.002.001.191207","telematics":{"generation":"3","platform":"1","tmsCenter":"1","billing":true},"versionNum":"GASOLINE","headUnitType":"2","hdRadio":"X40HAF","ampType":"NA","modem":{"meid":"352756079211901","mdn":"6574342729","iccid":"89148000005952739845"},"headUnitName":"daudio1","bluetoothRef":"19","headUnitDesc":"DAV2"}},"maintenance":{"nextServiceMile":5823.7197,"maintenanceSchedule":[7500,15000,22500,30000,37500,45000,52500,60000,67500,75000,82500,90000,97500,105000,112500]},"vehicleFeature":{"remoteFeature":{"lock":"1","unlock":"1","start":"3","stop":"1","scheduleCount":"2","inVehicleSchedule":"1","heatedSteeringWheel":"1","heatedSideMirror":"1","heatedRearWindow":"1","heatedSeat":"0","ventSeat":"0","alarm":"1","hornlight":"1","panic":"1","doorSecurity":"1","engineIdleTime":"1","separateHeatedAccessories":"0","windowSafety":"0"},"chargeFeature":{"batteryChargeType":"0","chargeEndPct":"0","immediateCharge":"0","cancelCharge":"0","evRange":"0","scheduleCount":"0","inVehicleSchedule":"0","offPeakType":"0","scheduleType":"0","chargeLevel":"0","scheduleConfig":"0","fatcWithCharge":"0"},"alertFeature":{"geofenceType":{"geofence":"1","entryCount":"5","exitCount":"1","inVehicleConfig":"0","minRadius":"1","maxRadius":"10","minHeight":"1","maxHeight":"10","minWidth":"1","maxWidth":"10","uom":"0"},"curfewType":{"curfew":"1","curfewCount":"21","inVehicleConfig":"0"},"speedType":{"speed":"1","speedCount":"21","inVehicleConfig":"0"},"valetType":{"valet":"1","valetParkingMode":"0","defaultRadius":"1","defaultRadiusUnit":"3","defaultInterval":"5","defaultIntervalUnit":"3","inVehicleConfig":"0"}},"vrmFeature":{"autoDTC":"1","scheduledDTC":"1","backgroundDTC":"1","manualDTC":"1","healthReport":"0","drivingScore":"1","gasRange":"1","evRange":"0","trip":"1"},"locationFeature":{"gpsStreaming":"0","location":"1","poi":"1","poiCount":"25","push2Vehicle":"1","wayPoint":"1","mapType":"1","surroundView":"0","svr":"1"},"userSettingFeature":{"usmType":"0","calendar":"0","valetParkingMode":"0","wifiHotSpot":"0","otaSupport":"0","digitalKeyOption":"0"}},"heatVentSeat":{},"billingPeriod":{"freeTrial":{"value":12,"unit":0},"freeTrialExtension":{"value":12,"unit":1},"servicePeriod":{"value":60,"unit":1}}},"lastVehicleInfo":{"vehicleNickName":"Niro","preferredDealer":"PA004","licensePlate":"","psi":"","customerType":1,"vehicleStatusRpt":{"statusType":"2","reportDate":{"utc":"20211229124008","offset":-8},"vehicleStatus":{"climate":{"airCtrl":false,"defrost":false,"airTemp":{"value":"75","unit":1},"heatingAccessory":{"steeringWheel":0,"sideMirror":0,"rearWindow":0}},"engine":false,"doorLock":true,"doorStatus":{"frontLeft":0,"frontRight":0,"backLeft":0,"backRight":0,"trunk":0,"hood":0},"lowFuelLight":false,"ign3":false,"transCond":true,"distanceToEmpty":{"value":438,"unit":3},"tirePressure":{"all":0,"frontLeft":0,"frontRight":0,"rearLeft":0,"rearRight":0},"dateTime":{"utc":"20211229124008","offset":-8},"syncDate":{"utc":"20211229121444","offset":-8},"batteryStatus":{"stateOfCharge":100,"sensorStatus":0,"deliveryMode":0},"sleepMode":true,"lampWireStatus":{"headLamp":{},"stopLamp":{},"turnSignalLamp":{}},"windowStatus":{},"engineOilStatus":false,"vehicleMovementHis":false,"engineRuntime":{"value":8,"unit":1},"valetParkingMode":0}},"location":{"coord":{"lat":4,"lon":-7,"alt":61.2,"type":0,"altdo":0},"head":179,"speed":{"value":0,"unit":1},"accuracy":{"hdop":0,"pdop":1},"syncDate":{"utc":"20211229121444","offset":-8}},"financed":true,"financeRegistered":false,"linkStatus":0}}]}}
        """
        url = self.API_URL + "cmm/gvi"

        body = {
            "vehicleConfigReq": {
                "airTempRange": "0",
                "maintenance": "1",
                "seatHeatCoolOption": "1",
                "vehicle": "1",
                "vehicleFeature": "1",
            },
            "vehicleInfoReq": {
                "drivingActivty": "0",
                "dtc": "0",
                "enrollment": "0",
                "functionalCards": "0",
                "location": "1",
                "vehicleStatus": "1",
                "weather": "0",
            },
            "vinKey": [vehicle_key],
        }
        response = await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )

        response_json = await response.json()
        return response_json["payload"]

    async def request_vehicle_data_sync(self, session_id: str, vehicle_key: str):
        """
        {"status":{"statusCode":0,"errorType":0,"errorCode":0,"errorMessage":"Success with response body"},"payload":{"vehicleStatusRpt":{"statusType":"1","reportDate":{"utc":"20211130173341","offset":-8},"vehicleStatus":{"climate":{"airCtrl":false,"defrost":false,"airTemp":{"value":"72","unit":1},"heatingAccessory":{"steeringWheel":0,"sideMirror":0,"rearWindow":0}},"engine":false,"doorLock":true,"doorStatus":{"frontLeft":0,"frontRight":0,"backLeft":0,"backRight":0,"trunk":0,"hood":0},"lowFuelLight":false,"evStatus":{"batteryCharge":false,"batteryStatus":79,"batteryPlugin":0,"remainChargeTime":[{"remainChargeType":1,"timeInterval":{"value":0,"unit":4}},{"remainChargeType":2,"timeInterval":{"value":0,"unit":4}},{"remainChargeType":3,"timeInterval":{"value":0,"unit":4}}],"drvDistance":[{"type":2,"rangeByFuel":{"evModeRange":{"value":213,"unit":3},"totalAvailableRange":{"value":213,"unit":3}}}],"syncDate":{"utc":"20211130165836","offset":-8},"targetSOC":[{"plugType":0,"targetSOClevel":80,"dte":{"type":2,"rangeByFuel":{"gasModeRange":{"value":0,"unit":3},"evModeRange":{"value":213,"unit":3},"totalAvailableRange":{"value":213,"unit":3}}}},{"plugType":1,"targetSOClevel":90,"dte":{"type":2,"rangeByFuel":{"gasModeRange":{"value":0,"unit":3},"evModeRange":{"value":213,"unit":3},"totalAvailableRange":{"value":213,"unit":3}}}}]},"ign3":true,"transCond":true,"tirePressure":{"all":0},"dateTime":{"utc":"20211130173341","offset":-8},"syncDate":{"utc":"20211130165836","offset":-8},"batteryStatus":{"stateOfCharge":87,"sensorStatus":0},"sleepMode":false,"lampWireStatus":{"headLamp":{},"stopLamp":{},"turnSignalLamp":{}},"windowStatus":{},"engineRuntime":{},"valetParkingMode":0}}}}
        """
        url = self.API_URL + "rems/rvs"
        body = {
            "requestType": 0  # value of 1 would return cached results instead of forcing update
        }
        await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )

    async def check_last_action_status(
        self, session_id: str, vehicle_key: str, xid: str
    ):
        url = self.API_URL + "cmm/gts"
        body = {"xid": xid}
        response = await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )
        response_json = await response.json()
        return all(v == 0 for v in response_json["payload"].values())

    async def lock(self, session_id: str, vehicle_key: str):
        url = self.API_URL + "rems/door/lock"
        response = await self._get_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url
        )
        return response.headers["Xid"]

    async def unlock(self, session_id: str, vehicle_key: str):
        url = self.API_URL + "rems/door/unlock"
        response = await self._get_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url
        )
        return response.headers["Xid"]

    async def start_climate(
        self, session_id: str, vehicle_key: str, set_temp, defrost, climate, heating
    ):
        url = self.API_URL + "rems/start"
        body = {
            "remoteClimate": {
                "airCtrl": climate,
                "airTemp": {
                    "unit": 1,
                    "value": str(set_temp),
                },
                "defrost": defrost,
                "heatingAccessory": {
                    "rearWindow": int(heating),
                    "sideMirror": int(heating),
                    "steeringWheel": int(heating),
                },
                "ignitionOnDuration": {
                    "unit": 4,
                    "value": 9,
                },
            }
        }
        response = await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )
        return response.headers["Xid"]

    async def stop_climate(self, session_id: str, vehicle_key: str):
        url = self.API_URL + "rems/stop"
        response = await self._get_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url
        )
        return response.headers["Xid"]

    async def start_charge(self, session_id: str, vehicle_key: str):
        url = self.API_URL + "evc/charge"
        body = {"chargeRatio": 100}
        response = await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )
        return response.headers["Xid"]

    async def stop_charge(self, session_id: str, vehicle_key: str):
        url = self.API_URL + "evc/cancel"
        response = await self._get_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url
        )
        return response.headers["Xid"]

    async def set_charge_limits(
        self, session_id: str, vehicle_key: str, ac_limit: int, dc_limit: int
    ):
        url = self.API_URL + "evc/sts"
        body = {
            "targetSOClist": [
                {
                    "plugType": 0,
                    "targetSOClevel": dc_limit,
                },
                {
                    "plugType": 1,
                    "targetSOClevel": ac_limit,
                },
            ]
        }
        response = await self._post_request_with_logging_and_errors_raised(
            session_id=session_id, vehicle_key=vehicle_key, url=url, json_body=body
        )
        return response.headers["Xid"]



import logging

import time
from aiohttp import ClientSession, ClientResponse, ClientError

from .errors import AuthError, RateError
from .util import clean_dictionary_for_logging

_LOGGER = logging.getLogger(__name__)

BASE_URL: str = "api.telematics.hyundaiusa.com"
LOGIN_API: str = "https://" + BASE_URL + "/v2/ac/"
API_URL: str = "https://" + BASE_URL + "/ac/v2/"


def request_with_logging(func):
    """
    {'errorSubCode': 'IDM_401_4', 'functionName': 'remoteVehicleStatus', 'errorSubMessage': 'Feature disabled as it is not part of subscription.', 'errorMessage': 'Feature disabled as it is not part of subscription.', 'errorCode': 401}
    {'errorSubCode': 'HT_504', 'systemName': 'HATA', 'functionName': 'findMyCar', 'errorSubMessage': 'HATA findMyCar service failed while performing the operation FindMyCar', 'errorMessage': "We're sorry, but we could not complete your request. Please try again later.", 'errorCode': 504, 'serviceName': 'FindMyCar'}
    """

    async def request_with_logging_wrapper(*args, **kwargs) -> ClientResponse:
        url = kwargs["url"]
        json_body = kwargs.get("json_body")
        if json_body is not None:
            _LOGGER.debug(
                f"sending {url} request with {clean_dictionary_for_logging(json_body)}"
            )
        else:
            _LOGGER.debug(f"sending {url} request")
        response = await func(*args, **kwargs)
        _LOGGER.debug(
            f"response headers:{clean_dictionary_for_logging(response.headers)}"
        )
        try:
            response_json = await response.json()
            _LOGGER.debug(
                f"response json:{clean_dictionary_for_logging(response_json)}"
            )
        except RuntimeError:
            response_text = await response.text()
            _LOGGER.debug(f"response text:{response_text}")
        if "errorCode" in response_json:
            if (
                response_json["errorCode"] == 502
                and response_json.get("errorSubCode", "") == "HT_534"
            ):
                raise RateError
            if (
                response_json["errorCode"] == 502
                and response_json.get("errorSubCode", "") == "IDM_401_1"
            ):
                raise AuthError
            raise ClientError(f"api error:{response_json}")
        return response

    return request_with_logging_wrapper


def _api_headers(
    username: str,
    pin: str,
    access_token: str = None,
    vehicle_vin: str = None,
    vehicle_regid: str = None,
    extra_headers: dict[str, str] = None,
) -> dict[str, str]:
    headers = {
        "content-type": "application/json;charset=UTF-8",
        "accept": "application/json, text/plain, */*",
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "en-US,en;q=0.9",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36",
        "host": BASE_URL,
        "origin": "https://" + BASE_URL,
        "referer": "https://" + BASE_URL + "/login",
        "from": "SPA",
        "to": "ISS",
        "language": "0",
        "offset": str(int(time.localtime().tm_gmtoff / 60 / 60)),
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-origin",
        "refresh": "false",
        "encryptFlag": "false",
        "brandIndicator": "H",
        "gen": "2",
        "username": username,
        "blueLinkServicePin": pin,
        "client_id": "m66129Bb-em93-SPAHYN-bZ91-am4540zp19920",
        "clientSecret": "v558o935-6nne-423i-baa8",
    }
    if extra_headers is not None:
        headers.update(extra_headers)
    if vehicle_regid is not None:
        headers["registrationId"] = vehicle_regid
    if access_token is not None:
        headers["accessToken"] = access_token
    if vehicle_vin is not None:
        headers["vin"] = vehicle_vin
    return headers


class UsHyundai:
    def __init__(self, client_session: ClientSession = None):
        if client_session is None:
            self.api_session = ClientSession(raise_for_status=False)
        else:
            self.api_session = client_session

    async def cleanup_client_session(self):
        await self.api_session.close()

    @request_with_logging
    async def _post_request_with_logging_and_errors_raised(
        self,
        username: str,
        pin: str,
        url: str,
        json_body: dict,
        access_token: str = None,
        vehicle_vin: str = None,
        vehicle_regid: str = None,
        extra_headers: dict[str, str] = None,
    ) -> ClientResponse:
        headers = _api_headers(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            vehicle_regid=vehicle_regid,
            extra_headers=extra_headers,
        )
        return await self.api_session.post(url=url, json=json_body, headers=headers)

    @request_with_logging
    async def _get_request_with_logging_and_errors_raised(
        self,
        username: str,
        pin: str,
        url: str,
        access_token: str = None,
        vehicle_vin: str = None,
        vehicle_regid: str = None,
        extra_headers: dict[str, str] = None,
    ) -> ClientResponse:
        headers = _api_headers(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            vehicle_regid=vehicle_regid,
            extra_headers=extra_headers,
        )
        return await self.api_session.get(url=url, headers=headers)

    async def login(
        self, username: str, password: str, pin: str
    ) -> tuple[str, str, float]:
        """
        {"access_token":"","refresh_token":"","expires_in":"1799","username":"example@example.net"}
        """
        url = LOGIN_API + "oauth/token"
        json_body = {"username": username, "password": password}
        response: ClientResponse = (
            await self._post_request_with_logging_and_errors_raised(
                username=username, pin=pin, url=url, json_body=json_body
            )
        )
        response_json = await response.json()
        access_token: str = response_json["access_token"]
        refresh_token: str = response_json["refresh_token"]
        expires_in: float = float(response_json["expires_in"])
        return access_token, refresh_token, expires_in

    async def get_vehicles(
        self, username: str, pin: str, access_token: str
    ) -> dict[str, any]:
        """
        { "enrolledVehicleDetails": [ { "packageDetails": [ { "assetNumber": "1-229*******75", "displayCategory": "Connected Care", "packageId": "1-5MAJBS", "term": "36", "renewalDate": "20241014000000", "packageType": "Connected Care", "startDate": "20211014000000" }, { "assetNumber": "1-229*******35", "displayCategory": "Remote", "packageId": "1-5MAJLE", "term": "36", "renewalDate": "20241014000000", "packageType": "Remote", "startDate": "20211014000000" } ], "vehicleDetails": { "svrStatus": "NONE", "dynamicBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/sel/exterior/base/twilight-black/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "remoteStartWakeupDays": "seven", "enrollmentDate": "20211014", "trim": "SEL", "modelCode": "SANTA FE HYBRID", "ubiCapabilityInd": "Y", "vin": "VIN", "enrollmentId": "NUM", "sideMirrorHeatCapable": "YES", "ownersuccession": "1", "odometer": "63", "nickName": "2022 SANTA FE HYBRID", "defaultBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/sel/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "evStatus": "N", "modelYear": "2022", "steeringWheelHeatCapable": "YES", "defaultDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/sel/exterior/base/default/Dashboard-01.png", "vehicleGeneration": "2", "starttype": "BUTTON", "sapColorCode": "S3B", "bluelinkEnabled": true, "odometerUpdateDate": "20211016155605", "fatcAvailable": "Y", "color": "BLACK", "maintSyncCapable": "NO", "brandIndicator": "H", "deviceStatus": "ENROLLED", "mapProvider": "HERE", "generalBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/general/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "interiorColor": "NNB", "accessoryCode": "D-AUDIO2", "nadid": "nadid", "mit": "mit", "regid": "H00003854255VKM8S*******8299", "blueLink": "Y", "waypointInd": "NO", "billingInd": "YEARLY", "dynamicDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/sel/exterior/base/twilight-black/Dashboard-01.png", "imat": "7500", "additionalVehicleDetails": { "combinedHeatSettingsEnable": "Y", "temperatureRange": "false", "tmuSleepMode": "No", "enableHCAModule": "Y", "remoteLockConsentForRemoteStart": "No", "calendarVehicleSyncEnable": "No", "dkEnrolled": "N", "icpAvntCapable": "N", "icpAACapable": "N", "evAlarmOptionInfo": "No", "remoteLockConsentForRemoteStartCapable": "No", "mapOtaAccepted": "N", "icpCPCapable": "N", "dkCapable": "N", "enableValetActivate": "N" }, "transmissiontype": "AUTO", "bluelinkEnrolled": true, "rearWindowHeatCapable": "YES", "preferredDealerCode": "NJ045", "hmaModel": "TMHE", "series": "SANTA FE HYBRID", "enrollmentStatus": "ACTIVE", "generalDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2021/santa-fe-hybrid/general/exterior/base/default/Dashboard-01.png", "userprofilestatus": "NA" }, "roleDetails": [ { "roleCode": "SD", "roleName": "Secondary Driver" } ], "responseHeaderMap": {} } ], "addressDetails": [ { "city": "R****", "postalCode": "07***6", "type": "PRIMARY", "region": "NJ" } ], "user": { "accountId": "47***9", "firstName": "B*****", "lastName": "******", "loginId": "b*****@*****.com", "prefix": "Mr.", "additionalUserDetails": { "userProfileUpdate": "N", "timezoneOffset": -4, "appRating": "N", "timezoneAbbr": "EDT", "otaAcceptance": "N" }, "tncFlag": "Y", "idmId": "U0***SE", "userId": "b*****@*****.com", "notificationEmail": "b*****@*****.com", "email": "b*****@*****.com" } }
        {"enrolledVehicleDetails":[{"packageDetails":[{"assetNumber":"1-*","displayCategory":"Connected Care","packageId":"1-NZ1IOB","term":"1","renewalDate":"20211215000000","packageType":"Connected Care","startDate":"20210315000000"},{"assetNumber":"1-21339595491","displayCategory":"Remote","packageId":"1-NZ1ISZ","term":"1","renewalDate":"20211215000000","packageType":"Remote","startDate":"20210315000000"}],"driverDetails":[{"driverAddressDetails":[{"city":"CITY","postalCode":"ZIP","type":"PRIMARY","region":"NC"}],"driver":{"accountId":"NUM","firstName":"NAME","lastName":"NAME","phonesOptIn":[],"tncId":"20","loginId":"EMAIL,"preferredDealerCode":"NC072","driverUserProfile":"N","phones":[],"idmId":"idmId","userId":"EMAIL","email":"lysawinstonmines1@gmail.com"}}],"vehicleDetails":{"svrStatus":"NONE","dynamicBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/3.8/exterior/base/parisian-gray/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","remoteStartWakeupDays":"four","enrollmentDate":"20190207","svdDay":"06","trim":"3.8","modelCode":"GENESIS","ubiCapabilityInd":"Y","vin":"VIN","enrollmentId":"NUM","sideMirrorHeatCapable":"NO","ownersuccession":"1","odometer":"105088","nickName":"Phat boy","defaultBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/3.8/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","evStatus":"N","modelYear":"2015","steeringWheelHeatCapable":"NO","defaultDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/3.8/exterior/base/default/Dashboard-01.png","vehicleGeneration":"2","starttype":"BUTTON","silhouette":"https://owners.genesis.com/content/dam/genesis/us/mygenesis/image/Not_Avaliable.png","sapColorCode":"V6S","bluelinkEnabled":true,"odometerUpdateDate":"20211130052616","fatcAvailable":"Y","color":"GRAY","maintSyncCapable":"NO","brandIndicator":"H","deviceStatus":"ENROLLED","setOffPeak":"0","mapProvider":"HERE","generalBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/general/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","interiorColor":"RRY","accessoryCode":"AVN 4.5","nadid":"2343180194","mit":"3750","regid":"regid","blueLink":"Y","waypointInd":"NO","billingInd":"MONTHLY","dynamicDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/3.8/exterior/base/parisian-gray/Dashboard-01.png","imat":"imat","additionalVehicleDetails":{"combinedHeatSettingsEnable":"Y","temperatureRange":"false","tmuSleepMode":"No","enableHCAModule":"Y","remoteLockConsentForRemoteStart":"No","calendarVehicleSyncEnable":"No","tmuSleepInMin":140.28333333333333,"dkEnrolled":"N","icpAvntCapable":"N","icpAACapable":"N","enableRoadSideAssitanceAAAModule":"Y","evAlarmOptionInfo":"No","remoteLockConsentForRemoteStartCapable":"No","mapOtaAccepted":"N","icpCPCapable":"N","dkCapable":"N","enableValetActivate":"N","energyConsoleCapable":"No"},"transmissiontype":"AUTO","bluelinkEnrolled":true,"setChargeSchedule":"0","rearWindowHeatCapable":"NO","preferredDealerCode":"NC072","hmaModel":"DH","series":"GENESIS","enrollmentStatus":"ACTIVE","generalDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2015/genesis/general/exterior/base/default/Dashboard-01.png","userprofilestatus":"NA"},"roleDetails":[{"roleCode":"OWN","roleName":"OWNER"},{"roleCode":"SUB","roleName":"SUBSCRIBER"}],"responseHeaderMap":{}}],"addressDetails":[{"city":"CITY","street":"LOC","postalCode":"ZIP","type":"PRIMARY","region":"NC"}],"emergencyContacts":[{"firstName":"NAME","lastName":"NAME","contactId":"NUM","phones":[{"number":"PHONE","type":"mobile","order":1}],"relationship":"spouse","email":"EMAIL"}],"user":{"lastName":"NAME","phonesOptIn":[{"number":"NUM","primaryPhoneIndicator":"YES","fccOptIn":"YES","type":"MOBILE"}],"loginId":"EMAIL","prefix":"Mr.","tncFlag":"N","phones":[{"number":"PHONE","type":"cell","order":1}],"userId":"EMAIL","notificationEmail":"EMAIL","accountId":"NUM","firstName":"NAME","additionalUserDetails":{"userProfileUpdate":"N","timezoneOffset":-5,"appRating":"N","timezoneAbbr":"EST","otaAcceptance":"N","telematicsPhoneNumber":"PHONE"},"idmId":"AD7BDED7","email":"EMAIL"}}
        {"enrolledVehicleDetails":[{"packageDetails":[{"assetNumber":"1-12729429635","displayCategory":"Connected Care","packageId":"1-5MAJBS","term":"35","renewalDate":"20220815000000","packageType":"Connected Care","startDate":"20190816000000"},{"assetNumber":"1-12729429695","displayCategory":"Remote","packageId":"1-5MAJLE","term":"35","renewalDate":"20220815000000","packageType":"Remote","startDate":"20190816000000"}],"vehicleDetails":{"svrStatus":"NONE","dynamicBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/quartz-white/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","remoteStartWakeupDays":"four","enrollmentDate":"20190816","svdDay":"15","trim":"SEL","modelCode":"SONATA","ubiCapabilityInd":"Y","vin":"xxxxxx","enrollmentId":"xxxxx","sideMirrorHeatCapable":"YES","ownersuccession":"1","odometer":"8001","nickName":"2019 SONATA","defaultBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","evStatus":"N","modelYear":"2019","steeringWheelHeatCapable":"YES","defaultDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/default/Dashboard-01.png","vehicleGeneration":"2","starttype":"BUTTON","silhouette":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/sedan/Dashboard-01.svg","sapColorCode":"W8","bluelinkEnabled":true,"odometerUpdateDate":"20211216131200","fatcAvailable":"Y","color":"WHITE","maintSyncCapable":"NO","brandIndicator":"H","deviceStatus":"ENROLLED","setOffPeak":"0","mapProvider":"GOOGLE","generalBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/general/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","interiorColor":"BB","accessoryCode":"D-AUDIO","nadid":"xxxxx","mit":"7500","regid":"xxxxxxx","blueLink":"Y","waypointInd":"NO","billingInd":"YEARLY","dynamicDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/quartz-white/Dashboard-01.png","imat":"7500","additionalVehicleDetails":{"combinedHeatSettingsEnable":"Y","temperatureRange":"false","tmuSleepMode":"Yes","enableHCAModule":"Y","hyundaiHome":"N","remoteLockConsentForRemoteStart":"No","calendarVehicleSyncEnable":"No","dkEnrolled":"N","icpAvntCapable":"N","icpAACapable":"N","enableRoadSideAssitanceAAAModule":"Y","evAlarmOptionInfo":"No","remoteLockConsentForRemoteStartCapable":"No","mapOtaAccepted":"N","icpCPCapable":"N","dkCapable":"N","enableValetActivate":"N","energyConsoleCapable":"No"},"transmissiontype":"AUTO","bluelinkEnrolled":true,"setChargeSchedule":"0","rearWindowHeatCapable":"YES","preferredDealerCode":"NY120","hmaModel":"LF","series":"SONATA","enrollmentStatus":"ACTIVE","generalDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/general/exterior/base/default/Dashboard-01.png","userprofilestatus":"NA"},"roleDetails":[{"roleCode":"OWN","roleName":"OWNER"},{"roleCode":"SUB","roleName":"SUBSCRIBER"}],"responseHeaderMap":{}}],"addressDetails":[{"city":"XXXXXXXXXX","street":"XXXXXX","postalCode":"XXXXX","type":"PRIMARY","region":"NY"}],"user":{"accountId":"XXXXXX","firstName":"FirstName","lastName":"LastName","phonesOptIn":[{"number":"XXXXXXXXXXX","primaryPhoneIndicator":"YES","fccOptIn":"YES","type":"MOBILE"}],"loginId":"example@example.net","additionalUserDetails":{"userProfileUpdate":"Y","timezoneOffset":-5,"appRating":"N","timezoneAbbr":"EST","otaAcceptance":"N","telematicsPhoneNumber":"xxxxxxxxxxxx"},"tncFlag":"N","phones":[{"number":"XXXXXXXXXX","type":"cell","order":1}],"idmId": ”XXXXXXXX","userId":"example@example.net","notificationEmail":"example@example.net","email":"example@example.net"}}
        { "addressDetails": [ { "city": "Some City", "postalCode": "55555", "region": "GT", "street": "1 Main St", "type": "PRIMARY" } ], "enrolledVehicleDetails": [ { "packageDetails": [ { "assetNumber": "1-12345678901", "displayCategory": "Connected Care", "packageId": "1-5OOOOS", "packageType": "Connected Care", "renewalDate": "20991231000000", "startDate": "20991231000000", "term": "36" }, { "assetNumber": "1-23456789012", "displayCategory": "Guidance", "packageId": "1-5OOOOH", "packageType": "Guidance", "renewalDate": "20991231000000", "startDate": "20991231000000", "term": "36" }, { "assetNumber": "1-34567890123", "displayCategory": "Remote", "packageId": "1-5OOOOE", "packageType": "Remote", "renewalDate": "20991231000000", "startDate": "20991231000000", "term": "99" } ], "responseHeaderMap": {}, "roleDetails": [ { "roleCode": "OWN", "roleName": "OWNER" }, { "roleCode": "SUB", "roleName": "SUBSCRIBER" } ], "vehicleDetails": { "accessoryCode": "WAVN 5.0", "additionalVehicleDetails": { "calendarVehicleSyncEnable": "No", "combinedHeatSettingsEnable": "Y", "dkCapable": "N", "dkEnrolled": "N", "dynamicSOCText": "Use the slider above to set a charge limit. Charging will stop when this battery level is reached. The limit cannot be set lower than 50% This setting will override all other charge settings if set.", "enableHCAModule": "Y", "enableRoadSideAssitanceAAAModule": "Y", "enableValetActivate": "N", "energyConsoleCapable": "No", "evAlarmOptionInfo": "No", "hyundaiHome": "N", "icpAACapable": "N", "icpAvntCapable": "N", "icpCPCapable": "N", "mapOtaAccepted": "N", "maxTemp": 92, "minTemp": 62, "remoteLockConsentForRemoteStart": "No", "remoteLockConsentForRemoteStartCapable": "No", "targetSOCLevelMax": 100, "temperatureRange": "true", "tmuSleepInMin": 150.5, "tmuSleepMode": "No" }, "billingInd": "YEARLY", "blueLink": "Y", "bluelinkEnabled": true, "bluelinkEnrolled": true, "brandIndicator": "H", "color": "GRAY", "defaultBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/ev-limited/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "defaultDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/ev-limited/exterior/base/default/Dashboard-01.png", "deviceStatus": "ENROLLED", "dynamicBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/ev-limited/exterior/base/electric-shadow/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "dynamicDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/ev-limited/exterior/base/electric-shadow/Dashboard-01.png", "enrollmentDate": "20990101", "enrollmentId": "1234567", "enrollmentStatus": "ACTIVE", "enrollmentType": "INDIVIDUAL", "evStatus": "E", "fatcAvailable": "Y", "generalBurgerMenu": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/general/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png", "generalDashboard": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2020/ioniq-electric/general/exterior/base/default/Dashboard-01.png", "hmaModel": "AEEV", "imat": "5000", "interiorColor": "T9Y", "maintSyncCapable": "NO", "mapProvider": "HERE", "mit": "5000", "modelCode": "IONIQ ELECTRIC", "modelYear": "2020", "nadid": "1234567890", "nickName": "2020 IONIQ ELECTRIC", "odometer": "20", "odometerUpdateDate": "20211223080848", "ownersuccession": "1", "preferredDealerCode": "GR100", "rearWindowHeatCapable": "NO", "regid": "H00001234567ABCDE89DE0FG123456", "remoteStartWakeupDays": "seven", "sapColorCode": "USS", "series": "IONIQ ELECTRIC", "setOffPeak": "1", "sideMirrorHeatCapable": "NO", "silhouette": "https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/sedan/Dashboard-01.svg", "starttype": "KEY", "steeringWheelHeatCapable": "NO", "svdDay": "27", "svrStatus": "NONE", "targetSOCLevel": "50", "transmissiontype": "AUTO", "trim": "EV LIMITED", "ubiCapabilityInd": "Y", "userprofilestatus": "NA", "vehicleGeneration": "2", "vin": "ABCD12EF3GH456789", "waypointInd": "NO" } } ], "user": { "accountId": "1234567", "additionalUserDetails": { "appRating": "N", "otaAcceptance": "N", "telematicsPhoneNumber": "8005551212", "timezoneAbbr": "EST", "timezoneOffset": -5, "userProfileUpdate": "N" }, "email": "someone@example.com", "firstName": "Someone", "idmId": "A1BCD2EF", "lastName": "Somesurname", "loginId": "someone@example.com", "notificationEmail": "someone@example.com", "phones": [ { "number": "8005551212", "order": 1, "type": "cell" } ], "phonesOptIn": [ { "fccOptIn": "NO", "number": "8005551212", "primaryPhoneIndicator": "YES", "type": "MOBILE" } ], "tncFlag": "N", "userId": "someone@example.com" } }
        """
        url = API_URL + "enrollment/details/" + username
        response: ClientResponse = (
            await self._get_request_with_logging_and_errors_raised(
                username=username, pin=pin, access_token=access_token, url=url
            )
        )
        response_json = await response.json()
        return response_json

    async def get_cached_vehicle_status(
        self, username: str, pin: str, access_token: str, vehicle_vin: str
    ) -> dict[str, any]:
        """
        {'hataTID': 'xxxxxxxxxxxx', 'vehicleStatus': {'dateTime': '2021-12-19T15:42:13Z', 'acc': False, 'trunkOpen': False, 'doorLock': True, 'defrostStatus': 'false', 'transCond': True, 'doorLockStatus': 'true', 'doorOpen': {'frontRight': 0, 'frontLeft': 0, 'backLeft': 0, 'backRight': 0}, 'airCtrlOn': False, 'airTemp': {'unit': 0, 'hvacTempType': 0, 'value': '01H'}, 'battery': {'batSoc': 70, 'batState': 0, 'sjbDeliveryMode': 0}, 'vehicleLocation': {'coord': {'lon': -xxxxxx, 'type': 0, 'lat': xxxxxxx}}, 'ign3': False, 'ignitionStatus': 'false', 'lowFuelLight': False, 'sideBackWindowHeat': 0, 'dte': {'unit': 3, 'value': 263.0}, 'engine': False, 'defrost': False, 'hoodOpen': False, 'airConditionStatus': 'false', 'steerWheelHeat': 0, 'tirePressureLamp': {'tirePressureWarningLampRearLeft': 0, 'tirePressureWarningLampFrontLeft': 0, 'tirePressureWarningLampFrontRight': 0, 'tirePressureWarningLampAll': 0, 'tirePressureWarningLampRearRight': 0}, 'trunkOpenStatus': 'false'}} 2021-12-21 21:43:19 DEBUG (SyncWorker_5) [custom_components.kia_uvo.HyundaiBlueLinkAPIUSA] kia_uvo - Get Vehicles Response {"enrolledVehicleDetails":[{"packageDetails":[{"assetNumber":"1-XXXXX","displayCategory":"Connected Care","packageId":"1-XXXXXX","term":"35","renewalDate":"20220815000000","packageType":"Connected Care","startDate":"20190816000000"},{"assetNumber":"1-XXXXXX","displayCategory":"Remote","packageId":"1-XXXXXX","term":"35","renewalDate":"20220815000000","packageType":"Remote","startDate":"20190816000000"}],"vehicleDetails":{"svrStatus":"NONE","dynamicBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/quartz-white/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","remoteStartWakeupDays":"four","enrollmentDate":"20190816","svdDay":"15","trim":"SEL","modelCode":"SONATA","ubiCapabilityInd":"Y","vin":"XXXXXXXXXX","enrollmentId":"XXXXXX","sideMirrorHeatCapable":"YES","ownersuccession":"1","odometer":"8001","nickName":"2019 SONATA","defaultBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","evStatus":"N","modelYear":"2019","steeringWheelHeatCapable":"YES","defaultDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/default/Dashboard-01.png","vehicleGeneration":"2","starttype":"BUTTON","silhouette":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/sedan/Dashboard-01.svg","sapColorCode":"W8","bluelinkEnabled":true,"odometerUpdateDate":"20211216131200","fatcAvailable":"Y","color":"WHITE","maintSyncCapable":"NO","brandIndicator":"H","deviceStatus":"ENROLLED","setOffPeak":"0","mapProvider":"GOOGLE","generalBurgerMenu":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/general/exterior/base/default/Dashboard-01.png/jcr:content/renditions/cq5dam.thumbnail.105.68.png","interiorColor":"BB","accessoryCode":"D-AUDIO","nadid":"XXXXXXXXX","mit":"7500","regid":"XXXXXXXX","blueLink":"Y","waypointInd":"NO","billingInd":"YEARLY","dynamicDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/sel/exterior/base/quartz-white/Dashboard-01.png","imat":"7500","additionalVehicleDetails":{"combinedHeatSettingsEnable":"Y","temperatureRange":"false","tmuSleepMode":"Yes","enableHCAModule":"Y","hyundaiHome":"N","remoteLockConsentForRemoteStart":"No","calendarVehicleSyncEnable":"No","dkEnrolled":"N","icpAvntCapable":"N","icpAACapable":"N","enableRoadSideAssitanceAAAModule":"Y","evAlarmOptionInfo":"No","remoteLockConsentForRemoteStartCapable":"No","mapOtaAccepted":"N","icpCPCapable":"N","dkCapable":"N","enableValetActivate":"N","energyConsoleCapable":"No"},"transmissiontype":"AUTO","bluelinkEnrolled":true,"setChargeSchedule":"0","rearWindowHeatCapable":"YES","preferredDealerCode":"XXXXXX","hmaModel":"LF","series":"SONATA","enrollmentStatus":"ACTIVE","generalDashboard":"https://owners.hyundaiusa.com/content/dam/hyundai/us/myhyundai/image/2019/sonata/general/exterior/base/default/Dashboard-01.png","userprofilestatus":"NA"},"roleDetails":[{"roleCode":"OWN","roleName":"OWNER"},{"roleCode":"SUB","roleName":"SUBSCRIBER"}],"responseHeaderMap":{}}],"addressDetails":[{"city":"XXXXXXXXXXXX","street":"XXXXXXXXX","postalCode":"XXXXX","type":"PRIMARY","region":"NY"}],"user":{"accountId":"XXXXXXX","firstName":"FirstName","lastName":"LastName","phonesOptIn":[{"number":"XXXXXXXXXX","primaryPhoneIndicator":"YES","fccOptIn":"YES","type":"MOBILE"}],"loginId":"example@example.net","additionalUserDetails":{"userProfileUpdate":"Y","timezoneOffset":-5,"appRating":"N","timezoneAbbr":"EST","otaAcceptance":"N","telematicsPhoneNumber":"XXXXXXX"},"tncFlag":"N","phones":[{"number":"XXXXXXX","type":"cell","order":1}],"idmId":"XXXXXXX","userId":"example@example.net","notificationEmail":"example@example.net","email":"example@example.net"}}
        """
        url = API_URL + "rcs/rvs/vehicleStatus"
        response: ClientResponse = (
            await self._get_request_with_logging_and_errors_raised(
                username=username,
                pin=pin,
                access_token=access_token,
                vehicle_vin=vehicle_vin,
                url=url,
            )
        )
        response_json = await response.json()
        return response_json

    async def get_location(
        self, username: str, pin: str, access_token: str, vehicle_vin: str
    ) -> dict[str, any]:
        url = API_URL + "rcs/rfc/findMyCar"
        response: ClientResponse = (
            await self._get_request_with_logging_and_errors_raised(
                username=username,
                pin=pin,
                access_token=access_token,
                vehicle_vin=vehicle_vin,
                url=url,
            )
        )
        response_json = await response.json()
        return response_json

    async def lock(
        self,
        username: str,
        pin: str,
        access_token: str,
        vehicle_vin: str,
        vehicle_regid: str,
    ) -> dict[str, any]:
        url = API_URL + "rcs/rdo/off"
        extra_headers = {"registrationId": vehicle_regid, "APPCLOUD-VIN": vehicle_vin}
        json_body = {"userName": username, "vin": vehicle_vin}
        response = await self._post_request_with_logging_and_errors_raised(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            url=url,
            json_body=json_body,
            extra_headers=extra_headers,
        )
        response_json = await response.json()
        return response_json

    async def unlock(
        self,
        username: str,
        pin: str,
        access_token: str,
        vehicle_vin: str,
        vehicle_regid: str,
    ) -> dict[str, any]:
        url = API_URL + "rcs/rdo/on"
        extra_headers = {"APPCLOUD-VIN": vehicle_vin}
        json_body = {"userName": username, "vin": vehicle_vin}
        response = await self._post_request_with_logging_and_errors_raised(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            vehicle_regid=vehicle_regid,
            url=url,
            json_body=json_body,
            extra_headers=extra_headers,
        )
        response_json = await response.json()
        return response_json

    async def start_climate(
        self,
        username: str,
        pin: str,
        access_token: str,
        vehicle_vin: str,
        vehicle_regid: str,
        set_temp,
        defrost,
        climate,
        heating,
        duration,
    ) -> dict[str, any]:
        url = API_URL + "rcs/rsc/start"
        json_body = {
            "Ims": 0,
            "airCtrl": int(climate),
            "airTemp": {"unit": 1, "value": set_temp},
            "defrost": defrost,
            "heating1": int(heating),
            "igniOnDuration": duration,
            "username": username,
            "vin": vehicle_vin,
        }
        response = await self._post_request_with_logging_and_errors_raised(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            vehicle_regid=vehicle_regid,
            url=url,
            json_body=json_body,
        )
        response_json = await response.json()
        return response_json

    async def stop_climate(
        self,
        username: str,
        pin: str,
        access_token: str,
        vehicle_vin: str,
        vehicle_regid: str,
    ) -> dict[str, any]:
        url = API_URL + "rcs/rsc/stop"
        response = await self._post_request_with_logging_and_errors_raised(
            username=username,
            pin=pin,
            access_token=access_token,
            vehicle_vin=vehicle_vin,
            vehicle_regid=vehicle_regid,
            url=url,
            json_body={},
        )
        response_json = await response.json()
        return response_json



# run with "python3 src/kia_hyundai_api/us_kia_web_stub.py"
import logging
import asyncio

from getpass import getpass
from pathlib import Path
import sys

path_root = Path(__file__).parents[2]
sys.path.append(str(path_root))
from src.kia_hyundai_api.us_kia_web import UsKiaWeb

logger = logging.getLogger("src.kia_hyundai_api")
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)


async def testing():
    api: UsKiaWeb = UsKiaWeb()
    username = input("Username: ")
    password = getpass()
    try:
        session_id, basic_info = await api.login(username=username, password=password)
        logger.debug(session_id)
        logger.debug(basic_info)
#        vehicles = await api.get_vehicles(session_id=session_id)
#        identifier = vehicles["vehicleSummary"][0]["vehicleIdentifier"]
#        key = vehicles["vehicleSummary"][0]["vehicleKey"]
#        await api.get_cached_vehicle_status(session_id=session_id, vehicle_key=key)
#        await api.lock(session_id=session_id, vehicle_key=key)
    finally:
        await api.cleanup_client_session()


asyncio.run(testing())



# run with "python3 src/kia_hyundai_api/ca_hyundai_stub.py"
import asyncio

from pathlib import Path
import sys

path_root = Path(__file__).parents[2]
sys.path.append(str(path_root))
from src.kia_hyundai_api.ca_stub import testing
from src.kia_hyundai_api import CaHyundai

asyncio.run(testing(CaHyundai()))
